@@ -4,26 +4,23 @@ use strict;
 use warnings;
 
 use Carp qw(croak);
-use Scalar::Util qw(weaken);
+use Scalar::Util qw(weaken refaddr);
 
 use JavaScript;
 
 my %Context;
+my %Runtime;
 
 sub new {
     my ($pkg, $runtime) = @_;
 
-    $pkg = ref $pkg || $pkg;
+    my $self = jsc_create($runtime->{_impl});
 
-    my $cx_ptr = jsc_create($runtime->{_impl});
+    my $ptr = $self->jsc_ptr;
     
-    my $self = bless { _impl => $cx_ptr }, $pkg;
-
-    $Context{${$cx_ptr}} = $self;
-    
-    weaken($Context{$cx_ptr});
-
-    $self->{runtime} = $runtime;
+    $Context{$ptr} = $self;
+    weaken($Context{$ptr});
+    $Runtime{$ptr} = $runtime;
     
     return $self;
 }
@@ -35,7 +32,7 @@ sub eval {
     my @caller = caller();
     $name ||= "$caller[0] line $caller[2]";
     
-    my $rval = jsc_eval($self->{_impl}, $source, $name);
+    my $rval = jsc_eval($self, $source, $name);
 
     return $rval;
 }
@@ -49,7 +46,7 @@ sub eval_file {
     my $source = <$in>;
     close($in);
 
-    my $rval = jsc_eval($self->{_impl}, $source, $file);
+    my $rval = jsc_eval($self, $source, $file);
 
     return $rval;
 }
@@ -57,13 +54,13 @@ sub eval_file {
 sub find {
     my ($self, $context) = @_;
 
-    $context = $$context if ref $context eq 'SCALAR';
+    my $ptr = ref $context ? $context->ptr : $context;
     
-    if (!exists $Context{$context}) {
+    if (!exists $Context{$ptr}) {
         croak "Can't find context $context";
     }
     
-    return $Context{$context};
+    return $Context{$ptr};
 }
 
 sub call {
@@ -71,13 +68,13 @@ sub call {
     my $function = shift;
     my $args     = [@_];
     
-    return jsc_call($self->{_impl}, $function, $args);
+    return jsc_call($self, $function, $args);
 }
 
 sub can {
     my ($self, $method) = @_;
 
-    return jsc_can($self->{_impl}, $method);
+    return jsc_can($self, $method);
 }
 
 # Functions for binding perl stuff into JS namespace
@@ -249,7 +246,7 @@ sub bind_class {
     # Flags
     my $flags = $args{flags};
     
-    jsc_bind_class($self->{_impl}, $name, $pkg, $cons, $fs, $static_fs, $ps, $static_ps, $flags);
+    jsc_bind_class($self, $name, $pkg, $cons, $fs, $static_fs, $ps, $static_ps, $flags);
     
     return;
 }
@@ -278,7 +275,7 @@ sub bind_value {
             next;
         }
         
-        jsc_bind_value($self->{_impl}, $parent,
+        jsc_bind_value($self, $parent,
                        $paths[$num], $num == $#paths ? $object : {});
     }
     
@@ -291,7 +288,7 @@ sub unbind_value {
     my @paths = split /\./, $name;
     $name = pop @paths;
     my $parent = join(".", @paths);
-    jsc_unbind_value($self->{_impl}, $parent, $name);
+    jsc_unbind_value($self, $parent, $name);
 }
 
 sub set_branch_handler {
@@ -299,23 +296,76 @@ sub set_branch_handler {
 
     $handler = _resolve_method($handler, 1);
 
-    jsc_set_branch_handler($self->{_impl}, $handler);
+    jsc_set_branch_handler($self, $handler);
 }
 
 sub compile {
     my $self = shift;
     my $source = shift;
 
-    my $script = JavaScript::Script->new($self->{_impl}, $source);
+    my $script = JavaScript::Script->new($self, $source);
     return $script;
 }
 
+sub get_version {
+    my ($self, $version) = @_;
+    return jsc_get_version($self);
+}
+
+sub set_version {
+    my ($self, $version) = @_;
+    jsc_set_version($self, $version);
+    1;
+}
+
+
+{
+    my %options_by_tag = (
+        strict  => 1,
+        xml     => 1 << 6,
+        jit     => 1 << 11,
+    );
+
+    sub get_options {
+        my ($self) = @_;
+        my $options = jsc_get_options($self);
+        return grep { $options & $options_by_tag{$_} } keys %options_by_tag;
+    }
+    
+    sub has_options {
+        my $self = shift;
+    
+        my %options = map { $_ => 1 } $self->get_options;
+        
+        !exists $options{$_} && return 0 for @_;
+
+        return 1;
+    }
+    
+    sub toggle_options {
+        my $self = shift;
+        
+        my $options = 0;
+        for (@_) {
+            $options |= 1 if exists $options_by_tag{lc $_};
+        }
+        
+        jsc_toggle_options($self, $options);
+        
+        1;
+    }
+}
+
 sub _destroy {
     my $self = shift;
-    return unless $self->{'_impl'};
-    delete $Context{${$self->{_impl}}};
-    jsc_destroy($self->{'_impl'} );
-    delete $self->{'_impl'};
+    return unless $self;
+    my $ptr = $self->jsc_ptr;
+    return unless exists $Context{$ptr};
+    delete $Context{$ptr};
+    jsc_destroy($self);
+
+    delete $Runtime{$ptr};
+    
     return 1;
 }
 
@@ -496,8 +546,55 @@ To remove the handler call this method with an undefined argument.
 
 The handler is called when a script branches backwards during execution, when a function returns and the end of the script. To continue execution the handler must return a true value. To abort execution either throw an exception or return a false value.
 
+=item get_version ( )
+
+Returns the runtime version of the context as a string, for exmaple C<1.7> or or C<ECMAv3>.
+
+=item set_version ( $version )
+
+Sets the runtime version of the context to that specified in the string I<$version>. Some features 
+such as C<let> and C<yield> might not be enabled by default and thus must be turned on by 
+specifying what JS version we're using.
+
+A list of these can be found at L<http://developer.mozilla.org/en/docs/JSVersion> but may vary 
+depending on the version of your runtime.
+
+=item get_options ( )
+
+Returns a list of the options currently enabled on the context.
+
+=item has_options ( OPTION, ... )
+
+Tests if the options are eneabled on the context.
+
+=item toggle_options ( OPTION, ... )
+
+Toggles the options on the context.
+
 =back
 
+=head2 OPTIONS
+
+A number of options can be set on contexts. The following are understood (case-insensitive):
+
+=over 4
+
+=item strict
+
+Warn on dubious practice.
+
+=item xml
+
+ECMAScript for XML (E4X) support: parse E<lt>!-- --E<gt> as a token, not backward compatible with the comment-hiding hack used in HTML script tags.
+
+=item jit
+
+Enable JIT compilation. Requires a SpiderMonkey with TraceMonkey.
+
+=back
+
+(Descriptions copied from jsapi.h and thus copyrighted under its license)
+
 =begin PRIVATE
 
 =head1 PRIVATE INTERFACE
@@ -556,6 +653,26 @@ Removes a new named property in I<parent>.
 
 Attaches a branch handler to the context. No check is made to see if I<handler> is a valid SVt_PVCV.
 
+=item jsc_get_version ( PJS_Context *context )
+
+Returns the version of the context as a string, for example "1.7"
+
+=item jsc_set_version ( PJS_Context *context, const char *version) 
+
+Set the version of the context to the one specified in version.
+
+=item jsc_get_options ( PJS_Context *context )
+
+Returns the options set on the undelying JSContext
+
+=item jsc_toggle_options ( PJS_Context *context, U32 options )
+
+Toggle the options on the underlying JSContext
+
+=item jsc_ptr ( PJS_Context *context )
+
+Return the address of the context for identification purposes.
+
 =back
 
 =end PRIVATE
